@@ -28,14 +28,11 @@
 #include <libwnck/screen.h>
 
 #include <glib.h>
-#include <gtk/gtkwidget.h>
+#include <gtk/gtk.h>
 #include <panel-applet.h>
 
 G_BEGIN_DECLS
 
-void        wncklet_set_tooltip         (GtkWidget  *widget,
-					 const char *tip);
-
 void        wncklet_display_about       (GtkWidget   *applet,
 					 GtkWidget  **dialog,
 					 const char  *name,
@@ -50,8 +47,8 @@ void        wncklet_display_about       (GtkWidget   *applet,
 
 void        wncklet_display_help        (GtkWidget  *widget,
 					 const char *doc_id,
-					 const char *filename,
-					 const char *link_id);
+					 const char *link_id,
+					 const char *icon_name);
 
 WnckScreen *wncklet_get_screen          (GtkWidget *applet);
 	
