@@ -14,6 +14,8 @@
 #include <glib/gi18n.h>
 #include <gdk/gdkx.h>
 
+#include <libpanel-util/panel-show.h>
+
 #include "button-widget.h"
 #include "drawer.h"
 #include "launcher.h"
@@ -154,7 +156,7 @@ panel_applet_recreate_menu (AppletInfo	*info)
 		menu->submenu =NULL;
 	}
 
-	gtk_widget_unref (info->menu);
+	g_object_unref (info->menu);
 	info->menu = panel_applet_create_menu (info);
 }
 
@@ -247,7 +249,8 @@ applet_callback_callback (GtkWidget      *widget,
 
 			panel_properties_dialog_present (drawer->toplevel);
 		} else if (strcmp (menu->name, "help") == 0) {
-			panel_show_help (screen, "user-guide.xml", "gospanel-18");
+			panel_show_help (screen,
+					 "user-guide", "gospanel-18", NULL);
 		}
 		break;
 	case PANEL_OBJECT_MENU:
@@ -469,8 +472,7 @@ panel_applet_create_menu (AppletInfo *info)
 
 	panel_widget = PANEL_WIDGET (info->widget->parent);
 
-	menu = g_object_ref (gtk_menu_new ());
-	gtk_object_sink (GTK_OBJECT (menu));
+	menu = g_object_ref_sink (gtk_menu_new ());
 
 	/* connect the show & deactivate signal, so that we can "disallow" and
 	 * "re-allow" autohide when the menu is shown/deactivated.
@@ -791,7 +793,15 @@ typedef struct {
 	guint            locked : 1;
 } PanelAppletToLoad;
 
+/* Each time those lists get both empty,
+ * panel_applet_queue_initial_unhide_toplevels() should be called */
 static GSList  *panel_applets_to_load = NULL;
+static GSList  *panel_applets_loading = NULL;
+/* We have a timeout to always unhide toplevels after a delay, in case of some
+ * blocking applet */
+#define         UNHIDE_TOPLEVELS_TIMEOUT_SECONDS 5
+static guint    panel_applet_unhide_toplevels_timeout = 0;
+
 static gboolean panel_applet_have_load_idle = FALSE;
 
 static void
@@ -818,6 +828,47 @@ panel_applet_on_load_queue (const char *id)
 	return FALSE;
 }
 
+/* This doesn't do anything if the initial unhide already happened */
+static gboolean
+panel_applet_queue_initial_unhide_toplevels (gpointer user_data)
+{
+	GSList *l;
+
+	if (panel_applet_unhide_toplevels_timeout != 0) {
+		g_source_remove (panel_applet_unhide_toplevels_timeout);
+		panel_applet_unhide_toplevels_timeout = 0;
+	}
+
+	for (l = panel_toplevel_list_toplevels (); l != NULL; l = l->next)
+		panel_toplevel_queue_initial_unhide ((PanelToplevel *) l->data);
+
+	return FALSE;
+}
+
+void
+panel_applet_stop_loading (const char *id)
+{
+	PanelAppletToLoad *applet;
+	GSList *l;
+
+	for (l = panel_applets_loading; l; l = l->next) {
+		applet = l->data;
+
+		if (strcmp (applet->id, id) == 0)
+			break;
+	}
+
+	/* this can happen if we reload an applet after it crashed,
+	 * for example */
+	if (l != NULL) {
+		panel_applets_loading = g_slist_delete_link (panel_applets_loading, l);
+		free_applet_to_load (applet);
+	}
+
+	if (panel_applets_loading == NULL && panel_applets_to_load == NULL)
+		panel_applet_queue_initial_unhide_toplevels (NULL);
+}
+
 static gboolean
 panel_applet_load_idle_handler (gpointer dummy)
 {
@@ -846,10 +897,17 @@ panel_applet_load_idle_handler (gpointer dummy)
 		g_slist_free (panel_applets_to_load);
 		panel_applets_to_load = NULL;
 		panel_applet_have_load_idle = FALSE;
+
+		if (panel_applets_loading == NULL) {
+			/* unhide any potential initially hidden toplevel */
+			panel_applet_queue_initial_unhide_toplevels (NULL);
+		}
+
 		return FALSE;
 	}
 
 	panel_applets_to_load = g_slist_delete_link (panel_applets_to_load, l);
+	panel_applets_loading = g_slist_append (panel_applets_loading, applet);
 
 	panel_widget = panel_toplevel_get_panel_widget (toplevel);
 
@@ -918,11 +976,15 @@ panel_applet_load_idle_handler (gpointer dummy)
 						 applet->locked,
 						 applet->position,
 						 applet->id);
+		break;
 	default:
+		g_assert_not_reached ();
 		break;
 	}
 
-	free_applet_to_load (applet);
+	/* Only the bonobo applets will do a late stop_loading */
+	if (applet->type != PANEL_OBJECT_BONOBO)
+		panel_applet_stop_loading (applet->id);
 
 	return TRUE;
 }
@@ -969,16 +1031,36 @@ panel_applet_compare (const PanelAppletToLoad *a,
 }
 
 void
-panel_applet_load_queued_applets (void)
+panel_applet_load_queued_applets (gboolean initial_load)
 {
-	if (!panel_applets_to_load)
-		return;
+	if (initial_load) {
+		if (!panel_applets_to_load) {
+			panel_applet_queue_initial_unhide_toplevels (NULL);
+			return;
+		} else {
+			/* Install a timeout to make sure we don't block the
+			 * unhiding because of an applet that doesn't load */
+			panel_applet_unhide_toplevels_timeout =
+				g_timeout_add_seconds (UNHIDE_TOPLEVELS_TIMEOUT_SECONDS,
+						       panel_applet_queue_initial_unhide_toplevels,
+						       NULL);
+		}
+	}
 
 	panel_applets_to_load = g_slist_sort (panel_applets_to_load,
 					      (GCompareFunc) panel_applet_compare);
 
 	if ( ! panel_applet_have_load_idle) {
-		g_idle_add (panel_applet_load_idle_handler, NULL);
+		/* on panel startup, we don't care about redraws of the
+		 * toplevels since they are hidden, so we give a higher
+		 * priority to loading of applets */
+		if (initial_load)
+			g_idle_add_full (G_PRIORITY_HIGH_IDLE,
+					 panel_applet_load_idle_handler,
+					 NULL, NULL);
+		else
+			g_idle_add (panel_applet_load_idle_handler, NULL);
+
 		panel_applet_have_load_idle = TRUE;
 	}
 }
@@ -1037,9 +1119,9 @@ panel_applet_save_position (AppletInfo *applet_info,
 	if (!immediate) {
 		if (!queued_position_source)
 			queued_position_source =
-				g_timeout_add (1000,
-					       (GSourceFunc) panel_applet_position_save_timeout,
-					       NULL);
+				g_timeout_add_seconds (1,
+						       (GSourceFunc) panel_applet_position_save_timeout,
+						       NULL);
 
 		if (!g_slist_find (queued_position_saves, applet_info))
 			queued_position_saves =
